package graph;
import java.util.*;
//import java.File;
import java.io.*;

public class AdjacencyMatrixGraph extends Graph{
	

	int[][] adjacencyMatrix;
	HashMap<Integer,Integer> idMap;
	HashMap<String,Integer> stringMap;
	HashMap<Integer,String> backMap;
	
	private int n;
	private int m;
	
	public AdjacencyMatrixGraph() {
   		this.n = 1;
   		adjacencyMatrix = new int [1] [1];
	}

	public AdjacencyMatrixGraph(int n) {
		this.n = n;
		adjacencyMatrix = new int [n][n];
	}
	
	public AdjacencyMatrixGraph(int n, double p) {
		Random ranGen = new Random();
		this.n = n;
		int m = 0;
		adjacencyMatrix = new int [n] [n];
		for (int i=0;i<n;i++){
			for (int j=i+1;j<n;j++){
				if (ranGen.nextDouble()<=p){
					adjacencyMatrix[i][j]=1;
					m++;
				}
			}
		}
		this.m =m;
	}
	
	public AdjacencyMatrixGraph(int n, int m) {
		Random ranGen = new Random();
		int i,j;
		this.n = n;
		this.m =m;
		adjacencyMatrix = new int [n] [n];
		int edgesLeft = m;
		while (edgesLeft>0){
			i = ranGen.nextInt(n);
			j = ranGen.nextInt(n);
			if (i!=j&&(!this.areAdjacent(i, j))){
				if (i<j) adjacencyMatrix[i][j]=1;
				else adjacencyMatrix[j][i]=1;
				m--;
			}
		}		
	}
	
	private void updateMap(int i){
		if (!idMap.containsKey(i)){
			idMap.put(i, n); //so far n vertices, spot 'n' is thus free
			n++;//added one new vertex
		}
	}
	
	public int getNodeForId(int i){
		return idMap.get(i);
	}
	
	public int getNodeForId(String name){
		return stringMap.get(name);
	}
	
	public AdjacencyMatrixGraph(File f){
		try{
			this.n = 0;
			BufferedReader reader = new BufferedReader(new FileReader(f));
			StringTokenizer tabber;
			idMap = new HashMap<Integer, Integer>();
			LinkedList<SimpleEdge> edgeList = new LinkedList<SimpleEdge>();
			String line = null;
			while ((line = reader.readLine()) != null) {
				tabber = new StringTokenizer(line);
				int v = Integer.valueOf(tabber.nextToken());
				int u = Integer.valueOf(tabber.nextToken());
				updateMap(v);
				updateMap(u);				
				edgeList.add(new SimpleEdge(idMap.get(u),idMap.get(v)));
			}
			System.out.println("Number of Graph Vertices:= "+n);
			adjacencyMatrix = new int [n] [n];
			for (SimpleEdge edge : edgeList){
				int u = edge.getU();
				int v = edge.getV();
				if (u<v){
					adjacencyMatrix[u][v]=1;
				}
				else{
					adjacencyMatrix[v][u]=1;
				}
			}
			this.m = edgeList.size();
			System.out.println("Number of Graph Edges: "+m);
		}
		catch (FileNotFoundException e) {
			
		}
		catch (IOException e) {
			
		}
	}
		
	@Override
	public boolean areAdjacent(int u, int v) {
		if(u<v && (adjacencyMatrix[u][v]==0)) return false;
		else if (u>v && adjacencyMatrix[v][u]==0) return false;
		else return true;
	}
	
	public int edgeNum(){
		return this.m;
	}

	public int vertexNum(){
		return this.n;
	}
	
	public void insertEdge(int i, int j){
		if (i<j) this.adjacencyMatrix[i][j] = 1;
		else if (j<i) this.adjacencyMatrix[j][i] = 1;
		if (i!=j) this.m++;
	}
	
	
	public String toString(){
		String result ="";
		for (int i=0;i<n;i++){
			for (int j=i+1;j<n;j++){
				if (backMap!=null)
				{if (this.areAdjacent(i, j)) result += backMap.get(i)+" " +backMap.get(j)+"\n";} 
			}
		}
		return result;
	}
	
	public AdjacencyMatrixGraph copyGraph(){
		AdjacencyMatrixGraph h = new AdjacencyMatrixGraph(this.n);
		for (int i =0;i<n;i++){
			for (int j =i+1;j<n;j++){
				if (this.areAdjacent(i, j))  h.insertEdge(i,j);
			}
		}
		return h;
	}
	
	public AdjacencyMatrixGraph copyGraph(int addVerts){
		AdjacencyMatrixGraph h = new AdjacencyMatrixGraph(this.n+addVerts);
		for (int i =0;i<n;i++){
			for (int j =i+1;j<n;j++){
				if (this.areAdjacent(i, j))  h.insertEdge(i,j);
			}
		}
		Random ranGen = new Random();
		double p = (double) 2*m / (double) this.n*(this.n-1);
		for (int i =0;i<h.vertexNum();i++){
			for (int j = Math.max(i+1,this.n);j<h.vertexNum();j++){
				if (p>=ranGen.nextDouble()){
					h.insertEdge(i,j);
				}
			}
		}	
		return h;
	}
	
	public void setStringMap(HashMap<String,Integer> map){
		this.stringMap = map;
	}
	
	public void setBackMap(HashMap<Integer,String> map){
		this.backMap = map;
	}
	
	public String getVertexString(int i){
		return backMap.get(i);
	}
	
	public HashMap<String,Integer> getStringMap(){
		return this.stringMap;
	}
	
	public AdjacencyMatrixGraph inducedGraphVertexDel(Set<Integer> del){
		AdjacencyMatrixGraph h = new AdjacencyMatrixGraph(this.n-del.size());
		int deleted=0;
		Map<Integer,Integer> newToOld = new HashMap<Integer, Integer>();
		for (int i =0;i<n;i++){
			if (del.contains(i)){
				deleted++;				
			}
			else {
				newToOld.put(i-deleted, i);
			}
		}
		//create new string maps
		HashMap<Integer, String> newBackMap = new HashMap<Integer, String>();
		HashMap<String, Integer> newStringMap = new HashMap<String, Integer>();
		for (int i =0;i<h.vertexNum();i++){
			//insert into new string maps
			newStringMap.put(this.getVertexString(newToOld.get(i)),i);
			newBackMap.put(i, this.getVertexString(newToOld.get(i)));
			//create new adjacencies
			for (int j =i+1;j<h.vertexNum();j++){
				int u = newToOld.get(i).intValue();
				int v = newToOld.get(j).intValue();
				if (this.areAdjacent(u, v))  h.insertEdge(i,j);
			}
		}
		h.setBackMap(newBackMap);
		h.setStringMap(newStringMap);
		return h;
	}
}
