package importer;

import graph.*;

import java.util.*;
import java.io.*;

public class ParseGraph {
	public enum graphFormat {ComplexAndTabGraph,TabGraph};
	graphFormat format;
	public ParseGraph(){}
	
	public ParseGraph(graphFormat gF){
		this.format = gF;
	}
	
	public Graph parse(String fileName){
		return new AdjacencyMatrixGraph(new File(fileName));
	}
	
	public ColorList parseColors(HashMap<String,List<String>> matches, AdjacencyMatrixGraph g, AdjacencyMatrixGraph h) throws IOException{
		HashMap<Integer,Set<Integer>>  clMap = new HashMap<Integer,Set<Integer>>();
		Set<Integer> cList;
		for (String prot1: matches.keySet()){
			//System.out.println("List for v:= " +prot1);			
			int v = g.getNodeForId(prot1);//first int denotes vertex that holds colorList (from g)
			cList = new HashSet<Integer>();
			for (String prot2: matches.get(prot1)){
				cList.add(h.getNodeForId(prot2));
			}
			clMap.put(v, cList);
		}
		return new ColorList(clMap);
	}
	
	public ColorList parseColors(String fileName, AdjacencyMatrixGraph g, AdjacencyMatrixGraph h) throws IOException{
		HashMap<Integer,Set<Integer>>  clMap = new HashMap<Integer,Set<Integer>>();
		Set<Integer> cList;
		BufferedReader reader = getReader(fileName);
		StringTokenizer tabber;
		String line = null;	
		while ((line = reader.readLine()) != null) {
			tabber = new StringTokenizer(line);
			int v = g.getNodeForId(Integer.valueOf(tabber.nextToken()));//first int denotes vertex that holds colorList (from g)
			cList = new HashSet<Integer>();
			while (tabber.hasMoreTokens()){
				cList.add(h.getNodeForId(Integer.valueOf(tabber.nextToken())));
			}
			clMap.put(v, cList);
		}
		return new ColorList(clMap);
	}
	
	public ListColoredAdjacencyMatrixGraph parseInducedGraph(Set<String> complex, String graphFile) throws IOException{
		//initializing the graph from the proteins
		HashMap<String,Integer> stringMap = new HashMap<String, Integer> ();
		HashMap<Integer, String> idMap = new HashMap<Integer, String> ();
		int i=0;
		for (String prot :complex) {
			stringMap.put(prot,i);
			idMap.put(i,prot);
			i++;
		}
		ListColoredAdjacencyMatrixGraph g = new ListColoredAdjacencyMatrixGraph(i);
		
		//reading edges from File
		BufferedReader reader = getReader(graphFile);
		StringTokenizer tabber;
		String prot1, prot2;
		String line = null;
		while ((line = reader.readLine()) != null) {
			tabber= new StringTokenizer(line);
			prot1 = tabber.nextToken();
			if (complex.contains(prot1)){
				prot2 = tabber.nextToken();
				if (complex.contains(prot2)){
					//System.out.println(prot1+" "+prot2);
					g.insertEdge(stringMap.get(prot1), stringMap.get(prot2));
				}
			}
		}
		g.setStringMap(stringMap);
		g.setBackMap(idMap);
		//System.out.println("Graph has "+g.vertexNum()+ " vertices.");
		//System.out.println("Graph has "+g.edgeNum()+ " edges.");
		return g;
	}
	
	public List<Complex> parseComplexList(String complexPath) throws IOException{
		List<Complex> complexList = new LinkedList<Complex>();
		BufferedReader reader = getReader(complexPath);
		StringTokenizer tabber;
		String name;
		String line = null;	
		while ((line = reader.readLine()) != null) {
			Set<String> complex = new HashSet<String> ();
			//System.out.println(line);
			tabber = new StringTokenizer(line,"\t");
			name = tabber.nextToken();			
			if (tabber.hasMoreTokens()){
				line = tabber.nextToken();			
				tabber = new StringTokenizer(line);
				while (tabber.hasMoreTokens()){
					complex.add(tabber.nextToken());
				}
				complexList.add(new Complex(name,complex));		
			}
		}
		return complexList;
	}

	public HashMap<String,List<String>> parseMatchList(Set<String> complex, String matchPath, double t) throws IOException{
		HashMap<String,List<String>> matches = new HashMap<String,List<String>>();
		for (String prot : complex){
			List<String> newList = new LinkedList<String> ();			
			matches.put(prot, newList);
		}
		BufferedReader reader = getReader(matchPath);
		StringTokenizer tabber;
		String line = null;	
		while ((line = reader.readLine()) != null) {
			tabber = new StringTokenizer(line);
			String prot1 = tabber.nextToken();
			String prot2 = tabber.nextToken();
			double edgeWeight = Double.valueOf(tabber.nextToken());
			//System.out.println(line+" "+edgeWeight);			
			if (edgeWeight<t){
				if (complex.contains(prot1)){
					matches.get(prot1).add(prot2);
				}
				else if (complex.contains(prot2)){
					matches.get(prot2).add(prot1);
				}				
			}
		}
		return matches;
	}
		
	private BufferedReader getReader(String path) throws IOException {		
			return new BufferedReader(new FileReader(new File(path)));		
	}
}
